﻿using System;
using System.Windows;
using System.Windows.Input;
using System.ComponentModel;
using System.Collections.ObjectModel;

namespace TimeLineSimpleSample
{
    using System.Linq;

    using Infragistics.Controls.Timelines;

    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : IMainWindow
    {
        public MainWindow()
        {
            InitializeComponent();      
            DataContext = new ViewModel();

            var model = DataContext as ViewModel;

            if (model != null)
            {
                model.View = this;
            }

            igDateTimeAxis.UnitType = DateTimeUnitType.Weeks;
            igDateTimeAxis.Unit = 1;
            igDateTimeAxis.Minimum = new DateTime(2014,1,1);
            igDateTimeAxis.Maximum = new DateTime(2014,12,30,23,59,59);
            igDateTimeAxis.AutoRange = false;
        }

        private void ButtonBase_OnClick(object sender, RoutedEventArgs e)
        {
            var model = DataContext as ViewModel;

            if (model != null)
            {
                model.CenterContent();
            }
        }

        public void UpdateTimeLineScroll(DateTime minItemDate, DateTime maxItemDate)
        {
            if (minItemDate == DateTime.MinValue || maxItemDate == DateTime.MinValue)
            {
                return;
            }

            // Check so that we are not out of scope
            if (minItemDate < igDateTimeAxis.ActualMinimum) minItemDate = igDateTimeAxis.ActualMinimum;
            if (maxItemDate > igDateTimeAxis.ActualMaximum) maxItemDate = igDateTimeAxis.ActualMaximum;

            // Calculate the relative sizes
            double span = (maxItemDate - minItemDate).Ticks;
            double minSpan = (minItemDate - igDateTimeAxis.ActualMinimum).Ticks;
            double range = (igDateTimeAxis.ActualMaximum - igDateTimeAxis.ActualMinimum).Ticks;
            double middle = minSpan + (span / 2);

            // Just to be save, don't divide by zero
            if (range == 0) return;

            double newScrollPosition = middle / range;
            
            // Add 0.05 to make it just a little larger then the content
            double newScrollScale = span / range + 0.05;

            // Update scroll position and scale to zoom in to content
            igDateTimeAxis.ScrollScale = newScrollScale;
            igDateTimeAxis.ScrollPosition = newScrollPosition;
        }
    }

    public interface IMainWindow
    {
        void UpdateTimeLineScroll(DateTime minItemDate, DateTime maxItemDate);
    }

    public class Data : INotifyPropertyChanged
    {
        private DateTime dueDate;

        public DateTime DueDate
        {
            get { return dueDate; }
            set { dueDate = value; PropChanged("DueDate"); }
        }

        private TimeSpan dur;

        public TimeSpan Duration
        {
            get { return dur; }
            set { dur = value; PropChanged("Duration"); }
        }


        private string title;

        public string Title
        {
            get { return title; }
            set { title = value; PropChanged("Title"); }
        }


        private string dec;

        public string Details
        {
            get { return dec; }
            set { dec = value; PropChanged("Details"); }
        }

        public DateTime Time
        {
            get
            {
                return DueDate.AddHours(Duration.TotalHours * -1); 
            }
        }


        public event PropertyChangedEventHandler PropertyChanged;

        public void PropChanged(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }

    }

    public class ViewModel : INotifyPropertyChanged
    {
        public ObservableCollection<Data> Data { get; set; }

        private double myScrollScale;
        public double MyScrollScale
        {
            get { return myScrollScale; }
            set { myScrollScale = value; PropChanged("MyScrollScale"); }
        }

        private double myScrollPosition;
        public double MyScrollPosition
        {
            get { return this.myScrollPosition; }
            set { this.myScrollPosition = value; PropChanged("MyScrollPosition"); }
        }

        public IMainWindow View { get; set; }

        public ViewModel()
        {
            Data = new ObservableCollection<Data>();
            Random r = new Random();

            for (int i = 0; i < 15; i++)
            {
                int data = r.Next(100) + 10;
                Data.Add(new Data
                {
                    Title = "Data " + i,
                    Details = "Data Details " + DateTime.Now.AddDays(data),
                    DueDate = DateTime.Now.AddDays(data),
                    Duration = new TimeSpan(5, 0, 0, 0)
                });
            }

            Data.Add(new Data
            {
                Title = "Data " + 15,
                Details = "Data Details " + DateTime.Now.AddDays(15),
                DueDate = DateTime.Now.AddDays(15),
                Duration = new TimeSpan(15, 0, 0, 0)
            });

            Data.Add(new Data
            {
                Title = "Data " + 16,
                Details = "Data Details " + DateTime.Now.AddDays(20),
                DueDate = DateTime.Now.AddDays(20),
                Duration = new TimeSpan(20, 0, 0, 0)
            });


            //Here you should cacluculate the values for the MyScroll and MyScrollPosition Proeperties.

            this.MyScrollScale = .75;
            this.MyScrollPosition = .5;
        }

        public event PropertyChangedEventHandler PropertyChanged;

        public void PropChanged(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }

        private ICommand _moreScaleCommand;
        public ICommand MoreScaleCommand
        {
            get
            {
                return _moreScaleCommand ?? (_moreScaleCommand = new CommandHandler(MoreScaleAction, true));
            }
        }
        
        public void MoreScaleAction()
        {
            var tempScale = this.MyScrollPosition + 0.1;
            this.MyScrollPosition = tempScale > 1 ? 1 : tempScale;
        }


        private ICommand _lessScaleCommand;
        public ICommand LessScaleCommand
        {
            get
            {
                return _lessScaleCommand ?? (_lessScaleCommand = new CommandHandler(LessScaleAction, true));
            }
        }

        public void LessScaleAction()
        {
            var tempScale = this.MyScrollPosition - 0.1;
            this.MyScrollPosition = tempScale < 0 ? 0 : tempScale;
        }

        private ICommand _moreScrollCommand;
        public ICommand MoreScrollCommand
        {
            get
            {
                return this._moreScrollCommand ?? (this._moreScrollCommand = new CommandHandler(MoreScrollAction, true));
            }
        }

        public void MoreScrollAction()
        {
            var tempScroll = this.MyScrollScale + 0.1;
            this.MyScrollScale = tempScroll > 1 ? 1 : tempScroll;
        }

        private ICommand _lessScrollCommand;
        public ICommand LessScrollCommand
        {
            get
            {
                return this._lessScrollCommand ?? (this._lessScrollCommand = new CommandHandler(LessScrollAction, true));
            }
        }

        public void LessScrollAction()
        {
            var tempScroll = this.MyScrollScale - 0.1;
            this.MyScrollScale = tempScroll < 0 ? 0 : tempScroll;
        }

        /// <summary>
        /// Find the earliest and the latest date amongst the timeline objects
        /// </summary>
        public void CenterContent()
        {
            var firstItem = Data.FirstOrDefault();

            if(firstItem == null) return;

            var minDate = firstItem.DueDate;
            var maxDate = firstItem.DueDate.AddSeconds(firstItem.Duration.TotalSeconds);

            foreach (var timeLineModel in Data.Where(timeLineModel => timeLineModel.Title != firstItem.Title))
            {
                if (timeLineModel.DueDate < minDate)
                {
                    minDate = timeLineModel.DueDate;
                }

                if (timeLineModel.DueDate.Add(timeLineModel.Duration) > maxDate)
                {
                    maxDate = timeLineModel.DueDate.Add(timeLineModel.Duration);
                }
            }
            
            View.UpdateTimeLineScroll(minDate, maxDate);
        }
    }

    public class CommandHandler : ICommand
    {
        private Action _action;
        private bool _canExecute;
        public CommandHandler(Action action, bool canExecute)
        {
            _action = action;
            _canExecute = canExecute;
        }

        public bool CanExecute(object parameter)
        {
            return _canExecute;
        }

        public event EventHandler CanExecuteChanged;

        public void Execute(object parameter)
        {
            _action();
        }
    }
}
